/* DupTree.h */

#ifndef __DUPTREE_H
#define __DUPTREE_H

#include "TInject.h"

_CLASSDEF(DuplicationsTree)

/*      */
class DuplicationsTree : public SpeciesTree
  {

  public :

    DuplicationsTree( RCString           aName="",
                      uint               nOfDuplications=0 );
    DuplicationsTree( RCDuplicationsTree aDuplicationssTree );
    DuplicationsTree( RCSpeciesTree      aSpeciesTree,
                      uint               nOfDuplications=0 );
    ~DuplicationsTree() {}

    uint numberOfDuplications()         const { return nOfDuplications; }
    uint numberOfDuplications( uint n )       { return nOfDuplications=n; }

    void     clearNumberOfDuplications  ();
    OK_ERROR defineNumberOfDuplications ( RCTreeInjector injector );
    OK_ERROR defineSumOfDuplications    ( RCTreeInjector injector );
    OK_ERROR defineNumberOfOmissions    ( RCTreeInjector injector );
    OK_ERROR defineSumOfOmissions       ( RCTreeInjector injector );
    OK_ERROR defineSumOfLosses          ( RCTreeInjector injector );

    RDuplicationsTree child( uint number )
      { return treeToDT(SpeciesTree::child(number)); }
    RCDuplicationsTree child( uint number ) const
      { return treeToDT(SpeciesTree::child(number)); }

    /*    Object'a */
    virtual classType   isA    () const { return duplicationsTreeClass; }
    virtual const char* nameOf () const { return "DuplicationsTree"; }
    virtual void printOn       ( ostream& os ) const;

    /*  "" Clonable */
    virtual Clonable* clone() const { return new DuplicationsTree(ME); }

    virtual RDuplicationsTree operator = ( RCDuplicationsTree aDupTree );
    virtual RTree             operator = ( RCTree             aTree    );

  private :

    uint nOfDuplications;

    OK_ERROR countDuplications ( RTreeInjection injection );
    OK_ERROR countOmissions    ( RTreeInjection injection );

    void countSumDuplications ();
    void countSumLosses       ();

    virtual int addChild( RCTree childToAdd );
    virtual int addChild( RCString childName=""  )
      { return addChild( DuplicationsTree( childName ) ); }

    static RDuplicationsTree  treeToDT( RTree  tree );
    static RCDuplicationsTree treeToDT( RCTree tree );

  };

inline DuplicationsTree::DuplicationsTree( RCString aName, uint dupNumber ) :
  SpeciesTree(aName),
  nOfDuplications(dupNumber)
  {}
inline DuplicationsTree::DuplicationsTree( RCDuplicationsTree aDupTree ) :
  SpeciesTree(aDupTree.name()),
  nOfDuplications(aDupTree.nOfDuplications)
  { ME = aDupTree; }
inline DuplicationsTree::DuplicationsTree( RCSpeciesTree aSpecTree, uint dupNumber ) :
  SpeciesTree(aSpecTree.name()),
  nOfDuplications(dupNumber)
  { ME = aSpecTree; }

inline int DuplicationsTree::addChild( RCTree childToAdd )
  {
    if ( childToAdd.contains( ME ) )
      return ERROR;

    return ( childToAdd.isA() == duplicationsTreeClass ?
             addChildPtr( new DuplicationsTree(treeToDT(childToAdd)) ) :
             addChildPtr( new DuplicationsTree(childToAdd) ) );
  }

inline RDuplicationsTree DuplicationsTree::treeToDT( RTree tree )
  {
    if ( tree.isA() != duplicationsTreeClass )
      cerr << "!   " << tree.nameOf()
           << "&  DuplicationsTree& !" << endl;
    return RDuplicationsTree(tree);
  }
inline RCDuplicationsTree DuplicationsTree::treeToDT( RCTree tree )
  {
    if ( tree.isA() != duplicationsTreeClass )
      cerr << "!   " << tree.nameOf()
	   << "&  DuplicationsTree& !" << endl;
    return RDuplicationsTree(tree);
  }

#endif  // __DUPTREE_H
