/* GeneTree.cpp */
/*
()
() **************************************************************************
() * -inline   GeneTree                                       *
() **************************************************************************
()
*/

#include "GeneTree.h"

/*
()       : whatSpecies();
()    :      ;
()   :  ;
()  :  .
()
*/
String GeneTree::whatSpecies( RCString aGene ) const
  {
    if ( isLeaf() )
      return ( (name()==aGene) ? spec : String() );

    String sp;
    for ( uint n=0; n<numberOfChildren(); n++ )
      if ( (sp=child(n).whatSpecies(aGene))!="" )
        return sp;

    return "";
  }

/*
()       : geneList()  ( );
()    :    ͣ ,   ;
()   : ---;
()  :    ͣ ,  .
()
*/
SrtArray GeneTree::geneList() const
  {
    SrtArray gList;

    if ( isLeaf() )
      gList.add( name() );

    for ( uint n=0; n<numberOfChildren(); n++ )
      gList.addElements( child(n).geneList() );

    return gList;
  }

/*
()       : speciesList()  ( );
()    :    ͣ ,   ;
()   : ---;
()  :    ͣ ,  .
()
*/
SrtArray GeneTree::speciesList() const
  {
    SrtArray specList;

    if ( isLeaf() && spec!="" )
      specList.add( spec );

    for ( uint n=0; n<numberOfChildren(); n++ )
      {
        SrtArray spList=child(n).speciesList();
        for ( int i=0; i<spList.size(); i++ )
          if ( !specList.contains(spList[i]) )
            specList.add( spList[i] );
      }

    return specList;
  }

/*
()       : gene()  ();
()    :     ;
()   :  ;
()  :   .
()
*/
RCGeneTree GeneTree::gene( RCString geneName ) const
  {
    if ( isLeaf() )
      return ( (name()==geneName) ? ME : RCGeneTree(nullTree) );

    PCGeneTree g;
    for ( uint n=0; n<numberOfChildren(); n++ )
      if ( (g=&child(n).gene(geneName)) != &nullTree )
        return *g;

    return RCGeneTree(nullTree);
  }

/*
()       : species()  ();
()    :      (,   1);
()   :  ;
()  :   .
()  : !!!         !!!
()
*/
RCGeneTree GeneTree::species( RCString specName ) const
  {
    if ( isLeaf() )
      return ( (spec==specName) ? ME : RCGeneTree(nullTree) );

    PCGeneTree g;
    for ( uint n=0; n<numberOfChildren(); n++ )
      if ( (g=&child(n).species(specName)) != &nullTree )
        return *g;

    return RCGeneTree(nullTree);
  }

/*
()       : cutGenet()  ( );
()    :    ,  ,   ң;
()   :  ;
()  : OK/ERROR (/).
()
*/
void GeneTree::cutGene( RCString geneName )
  {
    for ( uint n=0; n<numberOfChildren(); n++ )
      if ( child(n).hasChild() )
        {
          child(n).cutGene(geneName);
          if ( child(n).numberOfChildren() == 1 )
            {
              for ( uint m=0; m<child(n).numberOfChildren(); m++ )
                {
                  child(n).child(m).distanceFromParent(
                    child(n).child(m).distanceFromParent() +
                    child(n).distanceFromParent() );
                  addChild( child(n).child(m) );
                }
              removeChild(n);
            }
        }
      else
        if ( child(n).name() == geneName )
          removeChild(n);
  }

/*
()       : mergeOneSpeciesSubrees()  (  );
()    :     ,    ;
()   : ---;
()  : ---.
()
*/
void GeneTree::mergeOneSpeciesSubrees()
  {
    if ( isLeaf() )
      return;
  
    //    
    for ( uint n=0; n<numberOfChildren(); n++ )
      child(n).mergeOneSpeciesSubrees();
            
    if ( !hasChild() || !child(0).isLeaf() )
      return;

    String childSpec=child(0).spec;
    double sumLen=child(0).distanceFromParent();

    // ,       
    for ( uint n=1; n<numberOfChildren(); n++ )
      if ( !child(n).isLeaf() || child(n).spec!=childSpec )
        return;
      else
        sumLen+=child(n).distanceFromParent();

    //  
    spec=childSpec;
    name(child(0).name());
    //        
    sumLen /= numberOfChildren();
    sumLen += distanceFromParent();
    distanceFromParent(sumLen);
    //  
    removeChild(0);
    while ( hasChild() )
      name(name()+'+'+child(0).name()), removeChild(0);
  }

/*
()       : printOn( ostream outputStream )  (  ...);
()    :         ;
()   :   ;
()  : ---.
()
*/
void GeneTree::printOn( ostream& os ) const
  {
    if ( spec!="" )
      os << '[' << spec << ']';
    BioTree::printOn(os);
  }

RGeneTree GeneTree::operator=( RCGeneTree aGeneTree )
  {
    BioTree::operator=(aGeneTree);

    spec = aGeneTree.spec;

    return ME;
  }

RTree GeneTree::operator=( RCTree aTree )
  {
    BioTree::operator=(aTree);

    if ( aTree.isA() == geneTreeClass )
      spec = RCGeneTree(aTree).spec;

    return ME;
  }

