/* LexYY.h */

#ifndef __LEXYY_H
#define __LEXYY_H

#include <iostream>

using namespace std;

#include "YYDefs.h"

extern "C" int yywrap( void );

extern "C++" {
struct yy_buffer_state;
typedef int yy_state_type;

_CLASSDEF(yyFlexLexer)

class FlexLexer
  {

  public:

    virtual ~FlexLexer()  { }

    const char* YYText() { return yytext; }
    int         YYLeng() { return yyleng; }

    virtual void
                 yy_switch_to_buffer( struct yy_buffer_state* new_buffer ) = 0;
    virtual struct yy_buffer_state*
                 yy_create_buffer( istream* s, int size ) = 0;
    virtual void yy_delete_buffer( struct yy_buffer_state* b ) = 0;
    virtual void yyrestart( istream* s ) = 0;
    virtual int  yylex( YYSTYPE& ) = 0;
    int          yylex( YYSTYPE& yylval, istream* new_in, ostream* new_out=0 )
      { switch_streams( new_in, new_out );  return yylex( yylval ); }
    virtual void switch_streams( istream* new_in=0, ostream* new_out=0 ) = 0;
    int          lineno() const { return yylineno; }
    int          debug() const { return yy_flex_debug; }
    void         set_debug( int flag ) { yy_flex_debug = flag; }

  protected:

    char* yytext;
    int   yyleng;
    int   yylineno;
    int   yy_flex_debug;

  };
}

_CLASSDEF(yyFlexLexer)

class yyFlexLexer : public FlexLexer
  {

  public:

    yyFlexLexer( istream* arg_yyin = 0, ostream* arg_yyout = 0 );
    virtual ~yyFlexLexer();

    unsigned yycurline() { return linecount; }

    void         yy_switch_to_buffer( struct yy_buffer_state* new_buffer );
    struct       yy_buffer_state* yy_create_buffer( istream* s, int size );
    void         yy_delete_buffer( struct yy_buffer_state* b );
    void         yyrestart( istream* s );
    virtual int  yylex( YYSTYPE& );
    virtual void switch_streams( istream* new_in, ostream* new_out );

  protected:

    virtual int   LexerInput( char* buf, int max_size );
    virtual void  LexerOutput( const char* buf, int size );
    virtual void  LexerError( const char msg[] );
    void          yyunput( int c, char* buf_ptr );
    int           yyinput();
    void          yy_load_buffer_state();
    void          yy_init_buffer( struct yy_buffer_state* b, istream* s );
    void          yy_flush_buffer( struct yy_buffer_state* b );
    int           yy_start_stack_ptr;
    int           yy_start_stack_depth;
    int*          yy_start_stack;
    void          yy_push_state( int new_state );
    void          yy_pop_state();
    int           yy_top_state();
    yy_state_type yy_get_previous_state();
    yy_state_type yy_try_NUL_trans( yy_state_type current_state );
    int           yy_get_next_buffer();

    istream*                yyin;
    ostream*                yyout;
    struct yy_buffer_state* yy_current_buffer;
    char                    yy_hold_char;
    int                     yy_n_chars;
    char*                   yy_c_buf_p;
    int                     yy_init;
    int                     yy_start;
    int                     yy_did_buffer_switch_on_eof;
    yy_state_type           yy_last_accepting_state;
    char*                   yy_last_accepting_cpos;
    yy_state_type*          yy_state_buf;
    yy_state_type*          yy_state_ptr;
    char*                   yy_full_match;
    int*                    yy_full_state;
    int                     yy_full_lp;
    int                     yy_lp;
    int                     yy_looking_for_trail_begin;
    int                     yy_more_flag;
    int                     yy_more_len;
    int                     yy_more_offset;
    int                     yy_prev_more_offset;
    unsigned                linecount;
    PTVariableList          l;

  };

#endif	// __LEXYY_H

