/* Optimize.cpp */

#include "Optimize.h"

#include <math.h>    // fabs()
#include <time.h>
#include <stdio.h>   // remove()
//#include <unistd.h>  // access()

#include "LogOut.h"    //     log-
#include "SpTrAr.h"    //   
#include "TIQMax.h"    //  
#include "TreeOut.h"   //  
#include "TUtils.h"    //     
#include "OpenFile.h"  //     /
#include "ReadTree.h"  //     

OK_ERROR writeResults( RCSpeciesTreesArray bestSTrees );


/*     */
OK_ERROR optimizeGivenSpeciesTree( RCClArray gTrees, RSpeciesTree sTree )
  {

    clock();  //       (???)

    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );


    /*     ***********************/

    cout << "  ... " << flush;

    /*         */
    double initialQuality=TreeInjector(gTrees,sTree).inject();
    CHECK( initialQuality, "    ." );
    logOutInitialInjections( gTrees, sTree, initialQuality );

    cout << "Ok." << endl;

    /*     -  ,   */
    if ( GLOBAL_BOOLEAN("InjectOnly") == true )
      {
        if ( logAllowed )
          tlog << endl << " *   ." << endl;

        if ( GLOBAL_BOOLEAN("ShowInjections") == true )
          {
            IMPORT_STRING( resultFileName, "ResultFileName" );
    
            if ( logAllowed )
              tlog << " *     \"" << resultFileName << "\"."
                   << endl;
    
            ofstream resultFile;
            if ( !openFile( resultFile, resultFileName ) )
              return ERROR;
                        
            TreePrinter().printInjections( gTrees, sTree, resultFile );
    
            cout << "  ." << endl << endl;
            cout << "    \""
                 << resultFileName << "\"." << endl;
          }

        return OK;
      }


    /*   *******************************************/

    cout << " :" << endl;

    if ( logAllowed )
      tlog << " *  .\n";

    clock_t startTime=clock();  //    

    int numOfPasses=TreeInjectionQualityMaximizer(gTrees,sTree).maximize();
    CHECK( numOfPasses, ":-(" )

    clock_t endTime=clock();    //     

    cout << endl;


    /*     **********************************/

    if ( numOfPasses > 0 )  //   
      {
        /*     */
        double resultQuality=TreeInjector(gTrees,sTree).inject();
        CHECK( resultQuality, "    ." );
        logOutOptimizedInjections( gTrees, sTree, resultQuality );

        /*      */
        if ( GLOBAL_BOOLEAN( "DefineLengthsOnSpeciesTree" ) )
          CHECK( TreeInjectionQualityMaximizer(gTrees,sTree).defineLengths(),
                 "      ." )

        cout << endl;

        IMPORT_STRING( resultFileName, "ResultFileName" );

        if ( logAllowed )
          tlog << " *       \""
               << resultFileName << "\"." << endl;

        ofstream resultFile;
        if ( !openFile( resultFile, resultFileName ) )
          return ERROR;

        TreePrinter().phylipPrint( sTree, resultFile );

        cout << "   ." << endl << endl;
        cout << "      \""
             << resultFileName << "\"." << endl << endl;
        cout << "     : " << numOfPasses    << ".\n";
        cout << "        : " << initialQuality << ".\n";
        cout << "   : " <<  resultQuality << ".\n";

        if ( logAllowed )
          {
            tlog << endl << "   :" << endl;
            TreePrinter().printSpeciesTree( sTree, tlog );
          }
      }
    else  //    
      cout << "    ." << endl;

    /*       */
    if ( logAllowed )
      tlog << " *    < "
           << double(endTime-startTime+1)/CLOCKS_PER_SEC << " ." << endl;

    return OK;

  }

/*     */
OK_ERROR optimizeRandomSpeciesTrees( RCClArray gTrees, RSpeciesTree sTree,
                                     RCGenesTable gTable,
                                     RCIntMatrix  proximity )
  {

    clock();

    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );

    /*     ********************/

    IMPORT_INTEGER( numOfBestSTrees, "NumberOfResultSpeciesTrees" );
    IMPORT_BOOLEAN( resumeCalculations, "ResumeCalculations" );
    IMPORT_STRING( resultFileName, "ResultFileName" );

    SpeciesTreesArray bestSTrees(numOfBestSTrees);  //  
    SpeciesTree       bestSTree=sTree;              //   

    bool resultFileExists;
    {
      FILE * f = fopen( resultFileName, "r" );
      resultFileExists = ( f ? true : false );
      if ( f )
        fclose(f);
    }

//    if ( access( resultFileName, F_OK ) == OK )  // version with `access'
    if ( resultFileExists )  //   
      if ( resumeCalculations == false )  //  
        remove( resultFileName );  //    
      else // resumeCalculations == true
        {
          cout << "  ... " << flush;

          if ( logAllowed )
            tlog << " *    ." << endl;

          ClArray sTrees;
          CHECK( readSpeciesTrees(sTrees), "    ." )

          bool reinject=false;  //  ޣ  
          const String keyword("cost_");
          for ( int i=0; i<sTrees.size(); i++ ) //    
            {
              RSpeciesTree sTree((RSpeciesTree)(sTrees[i]));
              int offset=sTree.name().indexOf(keyword);
              if ( offset==sTree.name().length() )
                reinject=true;
              else
                offset+=keyword.length();

              String costAndNumber=sTree.name().substring(offset);
              double oldQuality=costAndNumber.substring( 0,
                                  costAndNumber.indexOf('x') );
              double realQuality=0;
              int    howMany=costAndNumber.substring(
                                  costAndNumber.indexOf('x')+1 );
              if ( howMany==0 )
                howMany=1;
              sTree.name("");

              /*         */
              if ( i>=3 && reinject==false )
                realQuality=oldQuality;
              else
                {
                  realQuality=TreeInjector(gTrees,sTree).inject();
                  CHECK( realQuality,
                         "     ." )
                  if ( reinject==false && fabs(oldQuality-realQuality) > 1e-10 )
                    reinject=true;
                }

              int place=bestSTrees.add( sTree, realQuality );
              if ( reinject=false &&  place!=i )
                {
                  cerr << "!      "
                          "  ! ";
                  reinject=true;
                }
              bestSTrees[place].howMany=howMany;
            }

          if ( bestSTrees.lastElementIndex() > 0 )
            bestSTree=RCSpeciesTree(sTrees[0]);

          cout << "Ok." << endl;
        }

    /*      */

    cout << "  ." << endl << endl;

    IMPORT_INTEGER( numOfRandSTrees, "NumberOfRandomSpeciesTrees" );
    IMPORT_INTEGER( autosavePeriod, "AutosavePeriod" );

    int bestTryNum=-1;  //   

    for ( int tryNum=0; tryNum<numOfRandSTrees; tryNum++ )
      {

        /*      */
        cout << "#" << String("0")*(String(numOfRandSTrees).length() -
                                    String(tryNum+1).length())
             << (tryNum+1) << '/' << numOfRandSTrees << " : " << flush;
        logOutNewTryHeader( tryNum );

        /*      */
        randomizeSpeciesTree( sTree, proximity, gTable );
        logOutNewRandomSpeciesTree(sTree);

        /*     *******************/

        /*         */
////        double initialQuality=TreeInjector(gTrees,sTree).inject();
//tlog << " injector'." << endl;
        TreeInjector injector(gTrees,sTree);
//tlog << " ." << endl;
        double initialQuality=injector.inject();
//tlog << "  ." << endl;
        CHECK( initialQuality, "    ." );
        logOutInitialInjections( gTrees, sTree, initialQuality );


        /*   ***************************************/

        if ( logAllowed )
          tlog << " *  ." << endl;

        int numOfPasses=TreeInjectionQualityMaximizer(gTrees,sTree).maximize();
        CHECK( numOfPasses, ":-(" )


        /*     ******************************/

        double resultQuality=initialQuality;
        if ( numOfPasses > 0 )  //   
          {
            /*     */
            resultQuality=TreeInjector(gTrees,sTree).inject();
            CHECK( resultQuality, "    ." );
            logOutOptimizedInjections( gTrees, sTree, resultQuality );

            if ( GLOBAL_BOOLEAN("DefineLengthsOnSpeciesTree") )
              CHECK( TreeInjectionQualityMaximizer(gTrees,sTree).defineLengths(),
                     "      ." );

            cout << " " << initialQuality << "-->" << resultQuality;
          }
        else  //    
          cout << "Q=" << initialQuality;

        /*        */
        int place=bestSTrees.add( sTree, resultQuality );
        if ( place == 0 && bestSTrees[0].howMany == 1 )
          {
            bestSTree  = sTree;
            bestTryNum = tryNum;
            if ( bestSTrees.lastElementIndex() > 0 )
              {
                cout << " -   !";
                if ( logAllowed )
                  tlog << endl << "    !" << endl;
                if ( GLOBAL_BOOLEAN("BeepOnNewMinimum") )
                  cerr << '\007';
              }
          }
        else if ( place >= 0 )  //     
          {
            cout << ": No." << (place+1) << "/" << bestSTrees.numPresent();
            if ( bestTryNum>=0 )
              cout << ", min(#" << (bestTryNum+1) << ")="
                   << bestSTrees[0].cost;
            else
              cout << ", min=" << bestSTrees[0].cost;
            if ( logAllowed )
              tlog << endl << "   " << (place+1)
                   << "-   " << bestSTrees.numPresent()
                   << "   ." << endl;
          }
        else // place < 0 :      
          if ( bestTryNum>=0 )
            cout << ", min(#" << (bestTryNum+1) << ")="
                 << bestSTrees[0].cost;
          else
            cout << ", min=" << bestSTrees[0].cost;

        cout << endl;

        /*     */
        if ( autosavePeriod!=0 && !((tryNum+1)%autosavePeriod) &&
             tryNum<numOfRandSTrees-1 )
          {
            CHECK( writeResults( bestSTrees ), "" );
            cout << "       ."
                 << endl;
          }

      } // for ( tryNum )

    sTree = bestSTree;

    logOutBestSpeciesTrees( sTree, bestSTrees, numOfBestSTrees );

    CHECK( writeResults( bestSTrees ), "" );

    cout << endl << "     \"" << resultFileName
         << "\"." << endl << endl;
    cout << "   : " << bestSTrees[0].cost << ".\n";

    return OK;

  }

OK_ERROR writeResults( RCSpeciesTreesArray bestSTrees )
  {
    IMPORT_STRING( resultFileName, "ResultFileName" );

    if ( GLOBAL_BOOLEAN( "LogAllowed" ) )
      tlog << endl << " *       \""
           << resultFileName << "\"." << endl;

    ofstream resultFile;
    if ( !openFile( resultFile, resultFileName ) )
      return ERROR;

    resultFile << bestSTrees;

    return OK;
  }
