/* ReadTree.cpp */

#include "ReadTree.h"

#include "TParser.h"
#include "GlSttngs.h"
#include "OpenFile.h"
#include "GeneTabl.h"
#include "GeneTree.h"
#include "SpecTree.h"

/*      */
OK_ERROR readGenesTable( RGenesTable gTable )
  {
    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );
    IMPORT_STRING( gTableFileName, "GenesTableFileName" );
    ifstream gTableFile;
    if ( ! openFile( gTableFile, gTableFileName ) )
      return ERROR;

    if ( logAllowed )
      tlog << " *     ." << endl;

    CHECK( gTable.readFrom(gTableFile), "" );

    if ( logAllowed )
      tlog << " *     ." << endl;

    return OK;  //    
  }

/*      */
int readGeneTrees( RClArray gTrees )
  {
    IMPORT_STRING( gTreeFileName, "GeneTreesFileName" );
    ifstream gTreeFile;
    if ( ! openFile( gTreeFile, gTreeFileName ) )
      return ERROR;

    if ( GLOBAL_BOOLEAN("LogAllowed") == true )
      tlog << " *   ." << endl;

    if ( TreeParser( gTreeFile, gTreeFileName ).getGeneTrees( gTrees ) != OK )
      return ERROR;

    if ( GLOBAL_BOOLEAN("LogAllowed") )
      tlog << " *     ." << endl
           << " *  " << gTrees.size() << "  ." << endl;

    return gTrees.size();
  }

/*      */
int readSpeciesTree( RSpeciesTree sTree )
  {
    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );
    IMPORT_STRING( sTreeFileName, "SpeciesTreeFileName" );
    ifstream sTreeFile;
    if ( ! openFile( sTreeFile, sTreeFileName ) )
      return ERROR;

    if ( logAllowed )
      tlog << " *   ." << endl;

    {
      ClArray tmpArray;
      if ( TreeParser(sTreeFile,sTreeFileName).getSpeciesTrees(tmpArray) != OK )
        if ( tmpArray.size() < 1 )
          return ERROR;
        else
          if ( logAllowed )
            tlog << " !      ." << endl;

      const int nOfGTrees=tmpArray.size();
      if ( nOfGTrees > 0 )
        {
          if ( nOfGTrees > 1 )
            if ( logAllowed )
              tlog << " ?  " << nOfGTrees << "   (?)"
                   << "  ." << endl;
          sTree = RSpeciesTree( tmpArray.firstElement() );
        }
    }

    if ( logAllowed )
      tlog << " *     ." << endl;

    return 1;  //    
  }

/*      */
int readSpeciesTrees( RClArray sTrees )
  {
    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );
    IMPORT_STRING( resultFileName, "ResultFileName" );
    ifstream sTreeFile;
    if ( ! openFile( sTreeFile, resultFileName ) )
      return ERROR;

    if ( logAllowed )
      tlog << " *   ." << endl;

    if ( TreeParser(sTreeFile,resultFileName).getSpeciesTrees(sTrees) != OK )
      if ( sTrees.size() < 1 )
        return ERROR;
      else
        if ( logAllowed )
          tlog << " !      ." << endl;

    if ( logAllowed )
      tlog << " *     ." << endl
           << " *  " << sTrees.size() << "  ." << endl;

    return sTrees.size();
  }
