/* Settings.cpp */

#include "Settings.h"

OK_ERROR Settings::set( RCString settingLine )
  {
    PSetting settingPtr=parseSetting( settingLine );
    if ( settingPtr != NULL )
      {
        OK_ERROR result=ERROR;
        if ( settingPtr->isRealSetting() )
          result=set( settingPtr->key(), settingPtr->realValue() );
        else if ( settingPtr->isStringSetting() )
          result=set( settingPtr->key(), settingPtr->stringValue() );
        else if ( settingPtr->isBooleanSetting() )
          result=set( settingPtr->key(), settingPtr->booleanValue() );
        else if ( settingPtr->isIntegerSetting() )
          result=set( settingPtr->key(), settingPtr->integerValue() );
        else
          cerr << "   : " << *settingPtr << endl;
        delete settingPtr;
        return result;
      }
    else
      return ERROR;
  }

OK_ERROR Settings::set( RCSettings settings )
  {
    for ( int i=0; i<=settings.lastElementIndex(); i++ )
      CHECK( set(settings[i]), "" )
    return OK;
  }

double Settings::getRealSettingValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isRealSetting() )
      {
        cerr << ":    " << endl
             <<  "  double " << key << endl;
//        cerr << ME << endl;
        return 0;
      }
    else
      return elementAt(i).realValue();
  }

String Settings::getStringSettingValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isStringSetting() )
      {
        cerr << ":    " << endl
             <<  "  String " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return elementAt(i).stringValue();
  }

bool Settings::getBooleanSettingValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isBooleanSetting() )
      {
        cerr << ":    " << endl
             <<  "  Boolean " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return elementAt(i).booleanValue();
  }

int Settings::getIntegerSettingValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isIntegerSetting() )
      {
        cerr << ":    " << endl
             <<  "  int " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return elementAt(i).integerValue();
  }

double Settings::getRealSettingDefaultValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isRealSetting() )
      {
        cerr << ":    " << endl
             <<  "  double " << key << endl;
//        cerr << ME << endl;
        return 0;
      }
    else
      return RCRealSetting(elementAt(i)).defaultValue();
  }

String Settings::getStringSettingDefaultValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isStringSetting() )
      {
        cerr << ":    " << endl
             <<  "  String " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return RCStringSetting(elementAt(i)).defaultValue();
  }

bool Settings::getBooleanSettingDefaultValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isBooleanSetting() )
      {
        cerr << ":    " << endl
             <<  "  Boolean " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return RCBooleanSetting(elementAt(i)).defaultValue();
  }

int Settings::getIntegerSettingDefaultValue( RCString key ) const
  {
    int i=indexOf(key);
    if ( indexOutOfRange(i) || !elementAt(i).isIntegerSetting() )
      {
        cerr << ":    " << endl
             <<  "  int " << key << endl;
//        cerr << ME << endl;
        return false;
      }
    else
      return RCIntegerSetting(elementAt(i)).defaultValue();
  }

void Settings::restoreDefaults()
  {
    for ( int i=0; i<size(); i++ )
      if ( isNotEmpty(i) )
        elementAt(i).restoreDefault();
  }

int Settings::indexOf( RCString key ) const
  {
    int i;
    for ( i=0; i<fBI; i++ )
      if ( elementAt(i)._key == key )
        break;
    return i;
  }

void Settings::printOn( ostream& os ) const
  {
    for ( int i=0; i<size(); i++ )
      if ( isNotEmpty(i) )
        os << elementAt(i).key() << " = "
           << elementAt(i).stringValue() << endl;
  }

PSetting Settings::parseSetting( RCString settingLine )
  {
    int indexOfEQS = settingLine.indexOf('=');
    int indexOfEOL = settingLine.indexOf('\n');
    String key   = (settingLine.substring( 0,            indexOfEQS )).trim();
    String value = (settingLine.substring( indexOfEQS+1, indexOfEOL )).trim();
    if ( key == "" )
      return 0;
    else
      return new StringSetting( key, value );
  }

