/* SpTrAr.h */

#ifndef __SpTrAr_h
#define __SpTrAr_h

#include "SpecTree.h"
#include "SrtArray.h"

_CLASSDEF(SpeciesTreesArray)

class SpeciesTreesArray : private SrtArray
  {

    _CLASSDEF(Element)

    class Element : public Sortable
      {

      public :

        const SpeciesTree tree;
        const double      cost;
        /* */ uint        howMany;

        Element( RCSpeciesTree sTree, double iCost=0 ) :
          tree(sTree),  cost(iCost), howMany(1)  {}
        Element( RCElement el ) : Sortable(),
          tree(el.tree),  cost(el.cost), howMany(1)  {}
        ~Element() {}

        bool isEqual          ( RCElement el ) const
          { return ( cost == el.cost && tree.isEqual(el.tree) ); }
        bool isLessThan       ( RCElement el ) const
          { return ( cost < el.cost ); }
        bool isGraiterThan    ( RCElement el ) const
          { return ( cost > el.cost ); }
        bool isNotLessThan    ( RCElement el ) const
          { return ( cost >= el.cost ); }
        bool isNotGraiterThan ( RCElement el ) const
          { return ( cost <= el.cost ); }

        /*    Object'a */
        virtual classType   isA     () const { return spTreesArrayElementClass; }
        virtual const char* nameOf  () const { return "Element"; }
        virtual bool        isEqual ( RCObject o ) const
          { return ( o.isA()==isA() && isEqual(RCElement(o)) ); }
        virtual void        printOn ( ostream& os ) const
          {
            os << tree << (tree.name()!=""?"_":"") << "cost_" << String(cost);
            if ( howMany != 1 ) os << 'x' << howMany;
            os << ';';
          }

        /*  "" Clonable */
        virtual PClonable clone() const { return new Element(ME); }

        /*  "" Sortable */
        virtual bool isLessThan       ( RCSortable s ) const
          { return ( isA()==s.isA() && isLessThan(RCElement(s)) ); }
        virtual bool isGraiterThan    ( RCSortable s ) const
          { return ( isA()==s.isA() && isGraiterThan(RCElement(s)) ); }
        virtual bool isNotLessThan    ( RCSortable s ) const
          { return ( isA()==s.isA() && isNotLessThan(RCElement(s)) ); }
        virtual bool isNotGraiterThan ( RCSortable s ) const
          { return ( isA()==s.isA() && isNotGraiterThan(RCElement(s)) ); }

      };

  public :

    SpeciesTreesArray( uint initialSize=0 )
      { if (initialSize) setSize(initialSize), lockSize(); }
    SpeciesTreesArray( RCSpeciesTreesArray A ) : SrtArray(A) {}
    ~SpeciesTreesArray() {}

    int  add( RCSpeciesTree sTree, double cost=0 );

    uint numPresent() const { return (lastElementIndex()+1); }

    void lockSize     () { SrtArray::lockSize(); }
    void unlockSize   () { SrtArray::unlockSize(); }
    bool sizeIsLocked () { return SrtArray::sizeIsLocked();  }

    int  lastElementIndex() const { return SrtArray::lastElementIndex(); }

    /*    Object'a */
    virtual classType   isA     () const { return speciesTreesArrayClass; }
    virtual const char* nameOf  () const { return "SpeciesTreesArray"; }
    virtual bool        isEqual ( RCObject o ) const
      { return ( o.isA() == isA() && isEqual(RCSpeciesTreesArray(o)) ); }

    /*  "" Clonable */
    virtual PClonable clone() const { return new SpeciesTreesArray(ME); }

    RElement  operator [] ( uint i )
      { return RElement(SrtArray::operator[](i)); }
    RCElement operator [] ( uint i ) const
      { return RCElement(SrtArray::operator[](i)); }

  };

/*         */
inline int SpeciesTreesArray::add( RCSpeciesTree sTree, double cost )
  {
    SpeciesTreesArray::Element newElement( sTree, cost );
    if ( contains(newElement) )
      {
        int i=firstIndexOf(newElement);
        ME[i].howMany++;
        return i;
      }
    else
      return SrtArray::add(newElement);
  }

inline ostream& operator << ( ostream& os, RCSpeciesTreesArray A )
  { return ( os << RCObject(A) ); }

#endif  // __SpTrAr_h
