/* TIQMax.cpp -    TreeInjectionQualityMaximizer */
/*
()
() **************************************************************************
() * -inline   TreeInjectionQualityMaximizer                  *
() **************************************************************************
()
*/

#include "TIQMax.h"   //   TreeInjectionQualityMaximizer

#include "TUtils.h"   //    
#include "TreeOut.h"  //    (   log-)
#include "LogOut.h"

/* "" #define    */
#define TIQMax TreeInjectionQualityMaximizer

/*
()       : maximize();
()    :     ;
()   : ---;
()  :    / ERROR.
()
*/
int TIQMax::maximize()
  {

    /*     */
    double initialQuality=inject();
    CHECK( initialQuality, "    ." )

    IMPORT_INTEGER( localOptimizationsOrder, "LocalOptimizationsOrder" );
    double  resultQuality=initialQuality;  //   
    int     numberOfChanges=0;  //     
    int     pass=0;  //  
    do  //    (  )
      {
        /*    */
        switch ( localOptimizationsOrder )
          {
          case 0 : numberOfChanges=optimize_Recoursive_UpToDown();    break;
          case 1 : numberOfChanges=optimize_Recoursive_DownToUp();    break;
          case 2 : numberOfChanges=optimize_ByLevels_UpToDown();      break;
          case 3 : numberOfChanges=optimize_ByLevels_DownToUp();      break;
          case 4 : numberOfChanges=optimize_Recoursive_BestToWorst(); break;
          default: cerr << ":   ."; return ERROR;
          }
        CHECK( numberOfChanges, ":  maximize()." )

        /* ,     */
        if ( numberOfChanges == 0 )  //   
          continue;

        /*     */
        resultQuality=inject();
        CHECK( resultQuality, String("     ") +
                              (pass+1) + "- ." )

        if ( GLOBAL_BOOLEAN("ShowProgressBar") == true )
          {
            if ( !((pass+1)%10) )
              cout << '#';
            else if ( !((pass+1)%5) )
              cout << '#';
            else
              cout << '#';
            cout.flush();
          }

        pass ++;
      }
    while ( numberOfChanges > 0 ); //    

    if ( GLOBAL_BOOLEAN("OrderSpeciesTreesBranches") == true )
      orderSpeciesTreeBranches( RSpeciesTree(target()) );

    return pass;

  }

/*
()       : optimize_ByLevels_DownToUp(...);
()    :  .. " " -;
()   :     ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_ByLevels_DownToUp( RSpeciesTree modTree )
  {
    int numberOfChanges=0;

    for ( int n=modTree.height()-2; n>=0; n-- )
      numberOfChanges+=optimize_Nth_Level(n);

    return numberOfChanges;
  }

/*
()       : optimize_ByLevels_UpToDown(...);
()    :  .. " " -;
()   :     ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_ByLevels_UpToDown( RSpeciesTree modTree )
  {
    int numberOfChanges=0;

    for ( uint n=0; n<=modTree.height()-2; n++ )
      numberOfChanges+=optimize_Nth_Level(n);

    return numberOfChanges;
  }

/*
()       : optimize_Recoursive_BestToWorst(...);
()    :  ..   " ";
()   :     ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_Recoursive_BestToWorst( RSpeciesTree modTree )
  {
    int numberOfChanges=0;

    if ( modTree.isRoot() )
      {
        double qualityDifference=optimize_Local(modTree);
        CHECK( qualityDifference, "" )
        numberOfChanges += (qualityDifference>0);
      }

    if ( modTree.hasChild() )
      {
        uint   bestChildNumber=0;
        double bestQualityDifference=0;
        for ( uint n=0; n<modTree.numberOfChildren(); n++ )
          {
            double qualityDifference=optimize_Local(modTree.child(n));
            CHECK( qualityDifference, "" )
            numberOfChanges += (qualityDifference>0);
            if ( qualityDifference>bestQualityDifference )
              bestChildNumber=n, bestQualityDifference=qualityDifference;
          }

        /*       */
        int childNumberOfChanges = 0;
        do
          {
            childNumberOfChanges =
              optimize_Recoursive_BestToWorst(modTree.child(bestChildNumber));
            CHECK( childNumberOfChanges, "" )
            numberOfChanges+=childNumberOfChanges;
          }
        while ( childNumberOfChanges>0 );

        /*       */
        for ( uint n=0; n<modTree.numberOfChildren(); n++ )
          if ( n!=bestChildNumber )
            {
              int childNumberOfChanges =
                optimize_Recoursive_BestToWorst(modTree.child(n));
              CHECK( childNumberOfChanges, "" )
              numberOfChanges+=childNumberOfChanges;
            }
      }

    return numberOfChanges;
  }

/*
()       : optimize_Recoursive_DownToUp(...);
()    :  ..   -;
()   :     ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_Recoursive_DownToUp( RSpeciesTree modTree )
  {
    int numberOfChanges=0;

    /*        */
    for ( uint n=0; n<modTree.numberOfChildren(); n++ )
      {
        int childNumberOfChanges=optimize_Recoursive_DownToUp(modTree.child(n));
        CHECK( childNumberOfChanges, "" )
        numberOfChanges+=childNumberOfChanges;
      }

    double qualityDifference=optimize_Local(modTree);
    CHECK( qualityDifference, "" )

    numberOfChanges+=(qualityDifference>0);

    return numberOfChanges;
  }

/*
()       : optimize_Recoursive_UpToDown(...);
()    :  ..   -;
()   :     ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_Recoursive_UpToDown( RSpeciesTree modTree )
  {
    double qualityDifference=optimize_Local(modTree);
    CHECK( qualityDifference, "" )

    int numberOfChanges=(qualityDifference>0);

    /*        */
    for ( uint n=0; n<modTree.numberOfChildren(); n++ )
      {
        int childNumberOfChanges=optimize_Recoursive_UpToDown(modTree.child(n));
        CHECK( childNumberOfChanges, "" )
        numberOfChanges+=childNumberOfChanges;
      }

    return numberOfChanges;
  }

/*
()       : optimize_nth_Level(...);
()    :  ..    ();
()   :        ;
()  :    / ERROR.
()
*/
int TIQMax::optimize_Nth_Level( RSpeciesTree modTree, uint level )
  {
    int numberOfChanges=0;

    if ( modTree.level()<level )
      for ( uint n=0; n<modTree.numberOfChildren(); n++ )
        {
          int childNumberOfChanges =
            optimize_Nth_Level( modTree.child(n), level );
          CHECK( childNumberOfChanges, "" )
          numberOfChanges += childNumberOfChanges;
        }
    else if ( modTree.level()==level )
      {
        double qualityDifference=optimize_Local(modTree);
        CHECK( qualityDifference, "" )
        numberOfChanges+=(qualityDifference>0);
      }

    return numberOfChanges;
  }

/*
()       : optimize_Local();
()    :       ;
()   : ---;
()  :       / ERROR.
()
*/
double TIQMax::optimize_Local( RSpeciesTree modTree )
  {
    TreeModifier modifier(modTree);  //   
    if ( modifier.isUnabled() )      //   ""
      return 0;                      //    (.,  )

    int bestVarNum=bestVariantNumber(modifier);  //  N  
    if ( bestVarNum == ERROR )
      {
        if ( GLOBAL_BOOLEAN("LogAllowed") == true )
          {
            tlog << "TIQMax:      \n"
                 << "          " << endl;
            TreePrinter().psGraphPrint( modTree, tlog );
          }
        return ERROR;
      }

    const double initQuality=inject();
    if ( bestVarNum != 0 )  //    
      modifier.makeModification(bestVarNum); //  
    const double newQuality=inject();

    double qualityDifference=(initQuality-newQuality)*(bestVarNum>0);

//if ( -1e9 < qualityDifference && qualityDifference < 1e9 )
//  tlog << String(qualityDifference) << ( qualityDifference>0 ? " > 0, " : " <= 0, " )
//       << bestVarNum << endl;

    return ( qualityDifference );
  }

/*
()       : bestVariantNumber( TreeModifier )  (  );
()    :        ;
()   :  ;
()  :      / ERROR.
()
*/
int TIQMax::bestVariantNumber( TreeModifier modifier )
  {
    int       bestVariantNumber=0;  //   
    double    bestQuality=0;        //    

    do
      {
        int n=modifier.modificationNumber();
        double quality=inject();  //  
        if ( quality == ERROR )  //     
          {  //   
            if ( GLOBAL_BOOLEAN("LogAllowed") == true )
              tlog << "TIQMax:    .\n";
            return ERROR;
          }
        if ( n!=0 && quality==bestQuality )
          if ( GLOBAL_BOOLEAN("LogAllowed") == true &&
               GLOBAL_BOOLEAN("WarnAmbiguousTransitions") == true )
            tlog << "(%-[) -  ." << endl;
        if ( n==0 || quality<bestQuality )  //   
          { bestVariantNumber = n; bestQuality = quality; }  // 

        modifier.makeModification();  //    
      }
    while ( modifier.modificationNumber() != 0 ); //    

    return bestVariantNumber;  //    
  }

